# 【Level1-2】今日の天気を表示（API利用）：都市を変更

## テーマ
1. Serviceクラスの取り扱いについて
2. 天気予報APIの利用について

## 演習課題の準備（開発環境について）
* この演習では、Codespacesと呼ばれるサービスを使って開発を行います。
* ブラウザ上で動作する開発環境です、インストール不要で使う事ができます。
* レベル0で実施した[手順](/Codespacesの実行手順.md)を参照して、演習課題の準備をしましょう。

## 演習内容
Serviceの一部を書き換えて自分の住んでいる都市の天気を表示しましょう。

# ヒント・解説
## Serviceクラス
PortalControllerのindexメソッド内の天気情報を取得している箇所を参照します。
```
/**
* level 1-2 天気取得都市の変更
*/
// 天気
Weather weather = weatherService.findWeather();
model.addAttribute("weather", weather);
```
天気の情報を得るために、`WeatherService`の`findWeather`メソッドが利用されます。  
取得した、weatherオブジェクトをビューに渡すことで、今日の天気、都市名、天気の画像を表示しています。  

![image](https://user-images.githubusercontent.com/32722128/189796088-2bbe6c93-694e-495f-84e3-96a4f4853fe4.png)  

`WeatherService`の`findWeather`メソッドを確認しましょう。
```
/**
* level 1-2 天気取得都市の変更
*
* https://weather.tsukumijima.net/ を参考に自分の住んでいる都市のcity codeをに変更する
*/
final String cityCode = "130010"; // 東京のCityCode
final String endpoint = "https://weather.tsukumijima.net/api/forecast";

final String url = endpoint + "?city=" + cityCode;

ResponseEntity<String> response = rest.getForEntity(url, String.class);
String json = response.getBody();

ObjectMapper objectMapper = new ObjectMapper();

JsonNode jsonNode = objectMapper.readTree(json);
Weather weather = new Weather();
weather.setCity(jsonNode.get("location").get("city").textValue());
weather.setForecast(jsonNode.get("forecasts").elements().next().get("telop").textValue());
weather
    .setImage(
        jsonNode.get("forecasts").elements().next().get("image").get("url").textValue());
return weather;
```
※例外処理は省いて、コードを抽出しました。  

今回は[天気予報API(livedoor天気互換)](https://weather.tsukumijima.net/)というAPIを使用しています。
パラメータとして地域別に定義されたID番号を指定する事で、指定された地域の天気情報をJSON形式で取得できます。  
[参考資料：JSON](https://wa3.i-3-i.info/word13798.html)  

例では、変数`cityCode`(地域別に定義されたID番号)に東京を示すコードである`130010`を指定し、
`https://weather.tsukumijima.net/api/forecast?city=130010`というURLを作成しURLに対してGETリクエストを行い、東京の天気のJSONデータを取得しています。  
取得したJSONから必要な値を取り出し、`weather`オブジェクトに格納し、コントローラーへ返しています。  
`weather`オブジェクトの`city`フィールドには、指定したIDで取得した地域名を、  
`forecast`フィールドには、天気を、`image`フィールドには、お天気の画像のURLを格納しています。  

では、現在「東京」に指定されている、地点を自分が住んでいる地域に変更しましょう。
地域別に定義されたIDは以下のページから探して下さい。  
[1次細分区定義表](https://weather.tsukumijima.net/primary_area.xml)

## ビューの動きについて
前回と同じように、indexメソッドは戻り値に`"index"`が指定されているため、`index.html`ファイルをテンプレートとして使用し、HTMLを生成します。
modelオブジェクトで渡した、属性名`weather`のデータについては以下の部分で使用されています。
```
<div class="row">
    <div class="col">
        <div class="card-title fs-4 fw-semibold">天気</div>
        <div class="card-subtitle text-disabled" th:text="${weather.forecast}">天気</div>
        <div class="card-text" th:text="${weather.city}">都市</div>
    </div>
    <div class="col text-end text-primary fs-4 fw-semibold">
        <img th:src="${weather.image}" alt="weather" width="70" height="70" />
    </div>
</div>
```
前回`th:text="${属性名}"`の形式で記載すると、コントローラーで設定した、値でタグ内の値が置き換えられるとお伝えしました。   
属性に指定した値が、オブジェクトである場合は、`th:text="${属性名.フィールド名}"`の形式で指定すると、指定されたフィールド名のgetterメソッドを呼び出し、フィールド値を取り出します。  
取り出した値で、タグ内の値が置き換えられます。

## 課題の提出
* 課題の提出は[課題提出の操作](/課題の提出手順.md)のコミット・プッシュ・プルリクエストを実施しましょう。
